\documentclass{article}
\usepackage[utf8x]{inputenc}
\usepackage{a4}
\usepackage{moreverb}
\usepackage{palatino}
\usepackage{url}
\newcommand{\OlxUm}{{O\hskip-.05ex\raisebox{-.4em}L\hskip-0.8ex X\raisebox{.4em}{{\footnotesize\sc um}}}}
%----------------------------------------------------------------------
\title{\OlxUm --- Um micro sistema de leilões em Haskell}
%----------------------------------------------------------------------
\author{
\\
	Universidade do Minho
\\
	LEI ---Licenciatura de Engenharia Informática
\\
	UC8201N6 --- Lab.\ de Informática I
\\
	Ano lectivo de 2012/13
}
%----------------------------------------------------------------------
\date{Nov.\ 2012}
%----------------------------------------------------------------------
%include polycode.fmt

\begin{document}
\maketitle

\begin{comment}
% ---------------------- advanced code, not for 1st year students ------------------------------
\begin{code}
{-# LANGUAGE TemplateHaskell #-}

module Auctions where

import Data.Aeson (decode, encode)
import Data.Aeson.TH (deriveJSON)
import qualified Data.ByteString as B
import qualified Data.ByteString.Lazy as BL
import Data.List (sort,nub)
\end{code}
% ----------------------------------------------------------------------------------------------
\end{comment}

\section{Tipos}
\OlxUm\ é um sítio para compra e venda em suporte internet que funciona em regime de
leilão. Começa por oferecer uma colecção de items para serem leiloados,
\begin{code}
data House = House {
               hrunning  :: Running,
               hfinished :: Finished
             } deriving (Show,Eq)
\end{code}
onde cada
\begin{code}
type Running  = [Auction]
\end{code}
é a lista dos ítens ainda por leiloar, e
\begin{code}
type Finished = [Auction]
\end{code}
contem os ítens cujo leilão já acabou.
Cada ítem do leilão é registado de acordo com a seguinte estrutura,
\begin{code}
data Auction = Auction {
                 actid     :: Int,
                 actowner  :: String,
                 actdesc   :: String,
                 actvalue  :: Int,
                 actbidder :: String
               } deriving (Show,Eq,Ord) 
\end{code}
a saber:
\begin{itemize}
\item |actid| --- o inteiro que identifica o ítem que está/foi leiloado
\item |actowner| --- o identificador do seu dono
\item |actdesc| --- a sua descrição
\item |actvalue| --- o seu valor
\item |actbidder| --- o identificador de quem o quer comprar
\end{itemize}

\subsection{Exemplos}
Dá-se de seguida um exemplo de leilão:
\begin{code}
myhouse = House [a2,a1] [a3] where
            a1 = Auction 1 "Peter" "TV" 60 ""
            a2 = Auction 2 "Mary" "laptop" 120 ""
            a3 = Auction 3 "John" "phone" 85 "Peter"
\end{code}
Este exemplo pode ser usado para testes.

\section{Acções}
Colocar novo ítem no leilão:
\begin{code}
auctionNew :: House -> String -> String -> Int -> House
auctionNew h owner desc value = 
  let newid = length (hrunning h) + length (hfinished h) + 1
      newa = Auction newid owner desc value ""
  in House (newa:hrunning h) (hfinished h)
\end{code}
Oferta para um particular item do leilão:
\begin{code}
auctionBid :: House -> Int -> String -> Int -> House
auctionBid h id bidder bid = 
  let r = filter ((/=id).actid) (hrunning h)
      curr = head (filter ((==id).actid) (hrunning h))
      newa = Auction (actid curr) (actowner curr) (actdesc curr) bid bidder
  in House (newa:r) (hfinished h)
\end{code}
Fechar o leilão de um ítem, dando-o como leiloado ou cancelado, caso não
tenha havido nenhum |actbidder| interessado em comprá-lo:
\begin{code}
auctionFinish :: House -> Int -> House
auctionFinish h id =
  let r = filter (\i -> (actid i) /= id) (hrunning h)
      curr = filter (\i -> (actid i) == id) (hrunning h)
  in House r (curr++(hfinished h))
\end{code}

\section{Tipos e funções auxiliares}
Função que dá o total já leiloado, total por leiloar e total sem oferta:
\begin{code}
tots :: House -> Tot
tots h = Tot f r l where
               r = foldr (+) 0 ( map actvalue (hrunning h) )
               f = foldr (+) 0 [ actvalue x | x <- hfinished h, actbidder x > "" ]
               l = foldr (+) 0 [ actvalue x | x <- hfinished h, actbidder x == "" ]
\end{code}
onde
\begin{code}
data Tot = Tot {tfin :: Int, trun :: Int, tlost :: Int} deriving Show
\end{code}
Função genérica para ordenar sequências de registos segundo um dos seus atributos:
\begin{code}
sortOn :: (Ord b) => (a -> b) -> [a] -> [a]
sortOn f l = [ l!!i | i <- map snd x ] where x = sort(zip (map f l) [0..])
\end{code}

\end{document}
